Attribute VB_Name = "Module_PointProx"
' Copyright  2018 Point Prox Inc.
'
'    This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program. If not, see <http://www.gnu.org/licenses/>.
'
'    Written by Alvin Lewis PMP, CSM President Point Prox Inc.
'    See the article at: <https://pointprox.com/...> for information about
'    how to install and use this software

Option Explicit

Sub RedHighlightSet()
Attribute RedHighlightSet.VB_ProcData.VB_Invoke_Func = "R\n14"
'
' Sets the highlight of the selected cells to red
'
' Keyboard Shortcut: Ctrl+Shift+E
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If

    With Selection.Interior
        .Pattern = xlSolid
        .PatternColorIndex = xlAutomatic
        .Color = 255
        .TintAndShade = 0
        .PatternTintAndShade = 0
    End With
End Sub

Sub GreenHighlightSet()
Attribute GreenHighlightSet.VB_ProcData.VB_Invoke_Func = "G\n14"
'
' Sets the selected cell to green highlight
'
' Keyboard Shortcut: Ctrl+Shift+G
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    With Selection.Interior
        .Pattern = xlSolid
        .PatternColorIndex = xlAutomatic
        .Color = 5287936
        .TintAndShade = 0
        .PatternTintAndShade = 0
    End With
End Sub

Sub GreyHighlightSet()
Attribute GreyHighlightSet.VB_ProcData.VB_Invoke_Func = "M\n14"
'
' Sets the selected cells to a mid grey
'
' Keyboard Shortcut: Ctrl+Shift+M
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    With Selection.Interior
        .Pattern = xlSolid
        .PatternColorIndex = xlAutomatic
        .Color = 14277081   '15% grey #d9d9d9
        .TintAndShade = 0
        .PatternTintAndShade = 0
    End With
End Sub

Sub YellowHighlightSet()
Attribute YellowHighlightSet.VB_ProcData.VB_Invoke_Func = "Y\n14"
'
' Sets the selected cells to yellow highlight
'
' Keyboard Shortcut: Ctrl+Shift+Y
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    With Selection.Interior
        .Pattern = xlSolid
        .PatternColorIndex = xlAutomatic
        .Color = 65535
        .TintAndShade = 0
        .PatternTintAndShade = 0
    End With
End Sub

Sub ClearHighlightSet()
Attribute ClearHighlightSet.VB_ProcData.VB_Invoke_Func = "C\n14"
'
' Clears the highlight and font colors of the selected cells
'
' Keyboard Shortcut: Ctrl+Shift+C
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    With Selection.Interior
        .Pattern = xlNone
        .TintAndShade = 0
        .PatternTintAndShade = 0
    End With
    
    With Selection.Font
        .ColorIndex = xlAutomatic
        .TintAndShade = 0
    End With
    
End Sub

Sub PasteSpecialValues()
Attribute PasteSpecialValues.VB_ProcData.VB_Invoke_Func = "V\n14"
'
' Performs a paste special of values
'
' Keyboard Shortcut: Ctrl+Shift+V
'
Dim bGoodCall As Boolean

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    bGoodCall = PasteSpecialValuesRun(False)
        
End Sub

Sub PasteValuesTransposed()
Attribute PasteValuesTransposed.VB_ProcData.VB_Invoke_Func = "T\n14"
'
' Pastes clipboard as transposed values
'
' Keyboard Shortcut: Ctrl+Shift+T
'
Dim bGoodCall As Boolean

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    bGoodCall = PasteSpecialValuesRun(True)

End Sub

Sub PasteSpecialWidthFormatValues()
Attribute PasteSpecialWidthFormatValues.VB_ProcData.VB_Invoke_Func = "A\n14"
'
' Pastes column width, format, and values
'
' Keyboard Shortcut: Ctrl+Shift+A
'
Dim bGoodCall As Boolean

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    bGoodCall = PasteSpecialExec(True, xlPasteColumnWidths, False)  'call for column widths
    If (bGoodCall) Then     'check if successful, if not may note be excel source on the clipboard
        bGoodCall = PasteSpecialExec(True, xlPasteFormats, False)
    End If
    bGoodCall = PasteSpecialValuesRun(False)
    
End Sub

Sub DeleteRow()
Attribute DeleteRow.VB_ProcData.VB_Invoke_Func = "D\n14"
'
' Deletes the entire row
'
' Keyboard Shortcut: Ctrl+Shift+D
'

Dim rngCur As Range

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngCur = Application.Selection
    If (rngCur Is Nothing) Then
        Exit Sub
    End If
    
    Selection.EntireRow.Delete
End Sub

Sub RowHeightReset()
Attribute RowHeightReset.VB_ProcData.VB_Invoke_Func = "H\n14"
'
' Sets the row height to fit text
'
' Keyboard Shortcut: Ctrl+Shift+R
'

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Selection.WrapText = True
    Selection.EntireRow.AutoFit
End Sub

Public Function StrFindCount(strToSearch As String, strToFind As String) As Integer
'
' Counts the number of occurances of strToFind in strToSearch
'   Called as a cell formula
'   strToSearch - the string to search within, usually a cell reference
'   strToFind - the string to count
'
Dim iCnt As Integer
Dim iLastFind As Integer
Dim iLen As Integer

    iCnt = 0
    StrFindCount = 0
    iLastFind = 1
    iLen = Len(strToSearch)
        
    Do While iLastFind > 0
            iLastFind = InStr(iLastFind, strToSearch, strToFind, vbTextCompare)
            If (iLastFind > 0) Then
                iCnt = iCnt + 1
                iLastFind = iLastFind + 1
            End If
            If (iLastFind > iLen) Then iLastFind = 0
    Loop
        
    StrFindCount = iCnt
    Exit Function
    
End Function

Sub PrintSelection()
Attribute PrintSelection.VB_ProcData.VB_Invoke_Func = "P\n14"
'
' Prints the current selection of cells
'
' Keyboard Shortcut: Ctrl+Shift+P
'
Dim rngCur As Range
Dim i As Integer

    On Error GoTo ERR_PrintSelection
    
    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngCur = Application.Selection
    
    rngCur.PrintOut
    
EXIT_PrintSelection:
    Exit Sub
    
ERR_PrintSelection:
    i = MsgBox("Somethng went wrong: " + Err.Description, vbCritical + vbOKOnly, "Bummer")
    Resume EXIT_PrintSelection
End Sub

Sub MakeUpper()
Attribute MakeUpper.VB_ProcData.VB_Invoke_Func = "U\n14"
'
' Makes the text of the current cell uppercase
'
' Keyboard Shortcut: Ctrl+Shift+U
'

Dim rngCur As Range
Dim i As Integer

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngCur = ActiveCell
    
    rngCur.Value = UCase(rngCur.Value)
    
End Sub

Sub MakeLower()
Attribute MakeLower.VB_ProcData.VB_Invoke_Func = "L\n14"
'
' Makes the text of the current cell lowercase
'
' Keyboard Shortcut: Ctrl+Shift+L
'
Dim rngCur As Range
Dim i As Integer

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngCur = ActiveCell
    
    rngCur.Value = LCase(rngCur.Value)
    
End Sub

Sub FindMax()
Attribute FindMax.VB_ProcData.VB_Invoke_Func = "X\n14"
'
' Finds first largest value in the selection and selects the cell containing it
'
' Keyboard Shortcut: Ctrl+Shift+X
'
Dim rngRtn As Range

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngRtn = FindMinMax(False)
    rngRtn.Select
    
End Sub

Sub FindMin()
Attribute FindMin.VB_ProcData.VB_Invoke_Func = "I\n14"
'
' Finds first smallest value in the selection and selects the cell containing it
'
' Keyboard Shortcut: Ctrl+Shift+I
'
Dim rngRtn As Range

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngRtn = FindMinMax(True)
    rngRtn.Select
    
End Sub


Sub SelectionParseToStringClip()
Attribute SelectionParseToStringClip.VB_ProcData.VB_Invoke_Func = "S\n14"
'
' Converts the current selection into a delimited string
'   and puts it on the clipboard. If the string is empty then it does nothing
'
' Keyboard Shortcut: Ctrl+Shift+S
'
Dim strDelim As String
Dim objDataObj As New MSForms.DataObject

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    strDelim = SelectionToStringExec("|")
    
    If (Len(strDelim) <> 0) Then
        objDataObj.SetText strDelim
        objDataObj.PutInClipboard
    End If

End Sub

Sub SelectToLastDataRow()
Attribute SelectToLastDataRow.VB_ProcData.VB_Invoke_Func = "W\n14"
'
' Sets the current selection to the first row to the last row with data
'   for the curently selected column
'
' Keyboard Shortcut: Ctrl+Shift+W
'
Dim rngLast As Range
Dim i As Integer

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngLast = FindLastDataCellRow()
    
    If (rngLast Is Nothing) Then
        i = MsgBox("Cannot find the end", vbOKOnly + vbInformation, "Dunno")
    Else
        Range(Cells(1, rngLast.Column), rngLast).Select
        rngLast.Activate
    End If
End Sub

Sub SelectToLastDataColumn()
Attribute SelectToLastDataColumn.VB_ProcData.VB_Invoke_Func = "N\n14"
'
' Sets the current selection from the first row to the last row with data
'   for the curently selected column
'
' Keyboard Shortcut: Ctrl+Shift+N
'
Dim rngLast As Range
Dim lLastRow As Long
Dim lFirstRow As Long
Dim i As Integer
Dim rngCur As Range

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngCur = Application.Selection
    
    i = 0
    lLastRow = rngCur.Rows.Count
    lFirstRow = rngCur.Row
    
    Set rngLast = FindLastDataCell(xlByColumns)    'get last cell
    
    If (rngLast Is Nothing) Then
        i = MsgBox("Cannot find the end", vbOKOnly + vbInformation, "Dunno")
    Else
        Range(Cells(lFirstRow, 1), Cells(lFirstRow + lLastRow - 1, rngLast.Column)).Select
        Cells(lFirstRow + lLastRow - 1, rngLast.Column).Activate
    End If

End Sub

Sub SetIncrementValue()
Attribute SetIncrementValue.VB_ProcData.VB_Invoke_Func = "Q\n14"
'
' Sets the last cell + 1 of the active cell's column to the max for the column + 1
'   If range values are numbers then will increment by 1
'   If range value are dates then will increment by a day
'   If value is cannot be interpreted as a number then the sub will set the cell to 1
'
' Keyboard Shortcut: Ctrl+Shift+Q
'
Dim rngActive As Range
Dim rngLast As Range
Dim vMax As Variant

    If Not RangeSetChk() Then
        Beep
        Exit Sub
    End If
    
    Set rngActive = ActiveCell
    
    Set rngLast = FindLastDataCellRow()     'get the last cell with data
    vMax = WorksheetFunction.Max(rngActive.EntireColumn)   'get the max for the column
    If (IsNumeric(vMax)) Then   'increment
        vMax = vMax + 1
    Else    'cannot be interpreted as numeric
        vMax = 1
    End If
    rngLast.Offset(1, 0).Select 'select the next cell
    ActiveCell.Value = vMax     'set the value

End Sub

Private Function PasteSpecialValuesRun(bTrans As Boolean) As Integer
'
' Utility that manages calling exec proc to paste special
' Attempts to call PasteSpecialExec anticipating that
'   the clipboard is loaded and contains a range of cells copied in Excel
' If that fails, it assumes the clipboard is loaded with text
' If fails again it assumes that clipboard is empty or
'   contains incompatible data and tells the user so
'
' Args:
'   bTrans = flag indicating to transpose values ro not

Dim i As Integer
Dim bWasGoodCall As Boolean
Dim iOp As XlPasteType

    bWasGoodCall = True
    iOp = xlPasteValues
    
    bWasGoodCall = PasteSpecialExec(bWasGoodCall, iOp, bTrans)
    
    If Not bWasGoodCall Then
        bWasGoodCall = PasteSpecialExec(bWasGoodCall, iOp, bTrans)
        If Not bWasGoodCall Then
            i = MsgBox("Nothing to paste or data not compatible", vbExclamation + vbOKOnly, "Copy Something Good")
        End If  'second call to paste special
    End If  'first call to paste special
    
    PasteSpecialValuesRun = bWasGoodCall
    
End Function

Private Function PasteSpecialExec(bState As Boolean, iOp As XlPasteType, bTrans As Boolean) As Boolean
' Utility function to paste special, returns true if succeded
'   else false if there was an error
'
' Args:
'   bState - Indicates to attempt to paste clipboard as an excel source (true) or as text (false)
'   iOp - the type of operation to perform (e.g. values, formats, and etc)
'   bTrans = flag indicating to transpose values ro not

Dim bGoodCall As Boolean

On Error GoTo ERR_PasteSpecialExec

    bGoodCall = True    'assume success
    If bState Then  'first call, assume clipboard is loaded with Excel copy
        Selection.PasteSpecial iOp, xlPasteSpecialOperationNone, False, bTrans
    Else    'call after error, assume clipboard has text
        ActiveSheet.PasteSpecial Format:="Text", NoHTMLFormatting:=True
    End If

EXIT_PasteSpecialExec:
    PasteSpecialExec = bGoodCall
    Exit Function
    
ERR_PasteSpecialExec:
    bGoodCall = False
    Resume EXIT_PasteSpecialExec
    
End Function

Private Function SelectionToStringExec(strDelim As String) As String
'
' Utility function converts the current selection into a delimited string and returns string
'   strDelim is the character to use as a delimiter. If the
'       trimed str is empty then defaults to a pipe char
'   If there is an error or nothing was selected then returns an empty string
'
' Args:
'   strDelim - the delimiter to use

Dim rngSelection As Range
Dim rngCell As Range
Dim strRtn As String
Dim i

On Error GoTo ERR_SelectionToStringExec

    Set rngCell = Nothing
    Set rngSelection = Application.Selection
    strRtn = ""
    strDelim = Trim(strDelim)
    
    If (Len(strDelim) = 0) Then 'default empty str to a pipe
        strDelim = "|"
    End If
    i = 0
    
    For Each rngCell In rngSelection
        If i = 0 Then
            strRtn = strRtn + rngCell.Value
        Else
            strRtn = strRtn + strDelim + rngCell.Value
        End If
        i = i + 1
    Next rngCell
    
EXIT_SelectionToStringExec:
    SelectionToStringExec = strRtn
    Exit Function
    
ERR_SelectionToStringExec:
    i = MsgBox("Error ocured: " + Err.Description, vbCritical + vbOKOnly, "Error Occured")
    strRtn = ""
    Resume EXIT_SelectionToStringExec
    
End Function    'SelectionToStringExec

Private Function FindMinMax(bFindMin) As Range
'
' Utility function finds first largest value in the selection and selects it
'
' Args:
'   bFindMin - flag indicating to find min or max, if true returns min else max

Dim rngSelection As Range
Dim rngCell As Range
Dim vMaxVal As Variant
Dim rngMaxCell As Range
Dim bIsRignt As Boolean
Dim i As Integer

    i = 0
    Set rngMaxCell = Nothing
    Set rngSelection = Application.Selection
    If (rngSelection Is Nothing) Then
        Exit Function
    End If
    
    bIsRignt = False
    
    For Each rngCell In rngSelection
        If (i = 0) Then     'is first row, prime the pump
            vMaxVal = rngCell.Value
            Set rngMaxCell = rngCell
        Else    'not first row
            If bFindMin Then
                bIsRignt = (rngCell.Value < vMaxVal)
            Else
                bIsRignt = (rngCell.Value > vMaxVal)
            End If
            
            If bIsRignt Then    'condition satisfied
                vMaxVal = rngCell.Value
                Set rngMaxCell = rngCell
            End If  'test if current cell value > then last max
        End If  'first row test
        i = i + 1
    Next rngCell
    
    Set FindMinMax = rngMaxCell
    
End Function

Private Function FindLastDataCellRow() As Range
'
' Func returns the last cell where the current column has data
'   Returns nothing if not found
'

Dim lRow As Long
Dim lCurCol As Long
Dim i As Integer
Dim rngLast As Range
Dim rngCur As Range

    Set rngCur = Application.Selection
    If (rngCur Is Nothing) Then
        Exit Function
    End If
    
    lRow = 0
    lCurCol = ActiveCell.Column
    
    Set rngLast = FindLastDataCell(xlByRows)
    
    If (rngLast Is Nothing) Then
        Exit Function
    End If
    
    lRow = rngLast.Row
    
    Do While (lRow > 0)     'loop up to find row with data
        'If (Range(Cells(lRow, lCurCol).Address).Value <> "") Then   'have to do this because no short circut AND in vba
        If (Cells(lRow, lCurCol).Value <> "") Then
            Exit Do 'found last with a value
        End If
        
        lRow = lRow - 1 'decrement to check next one
    Loop
    
    If (lRow = 0) Then      'return the first row
        lRow = 1
    End If
    
    Set FindLastDataCellRow = Cells(lRow, lCurCol)
    
End Function

Private Function FindLastDataCell(enOrder As XlSearchOrder) As Range
'
' Utility function returns the last cell where either the row or column has data
'   Returns nothing if not found
' Args:
'   enOrder: the search order (xlByRows or xlByColumns)
'
Dim rngRtn As Range
Dim i As Integer

On Error GoTo ERR_FindLastDataCell

    Set rngRtn = Cells.Find("*", , , , enOrder, xlPrevious) 'get the last indicated row or column
    
EXIT_FindLastDataCell:
    Set FindLastDataCell = rngRtn
    Exit Function
    
ERR_FindLastDataCell:
    i = MsgBox("Error on finding the end: " + Err.Description, vbOKOnly + vbCritical, "Error")
    Set rngRtn = Nothing
    Resume EXIT_FindLastDataCell
End Function

Private Function RangeSetChk() As Boolean
'
' Utility function checks to make sure the selection set
'   Returns true is selection is not nothing else false
'
Dim rngCur As Range

    Set rngCur = Application.Selection
    RangeSetChk = Not (rngCur Is Nothing)
End Function

